'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildFieldConfigArgumentMap = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = buildFieldConfigMap;

var _buildASTSchema = require('graphql/utilities/buildASTSchema');

var _produceType = require('./produceType');

var _produceType2 = _interopRequireDefault(_produceType);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const buildFieldConfigArgumentMap = exports.buildFieldConfigArgumentMap = (fieldAST, types) => fieldAST.arguments.reduce((map, inputValueAST) => {
  const type = (0, _produceType2.default)(inputValueAST.type, types);
  const defaultValue = inputValueAST.defaultValue;
  const description = (0, _buildASTSchema.getDescription)(inputValueAST);
  const argumentConfig = { type };
  if (defaultValue !== null) argumentConfig.defaultValue = type.parseLiteral(defaultValue);
  if (description) argumentConfig.description = description;
  return _extends({}, map, { [inputValueAST.name.value]: argumentConfig });
}, {});

function buildFieldConfigMap(objectAST, configMap = {}, types) {
  return objectAST.fields.reduce((map, fieldAST) => {
    const name = fieldAST.name.value;
    const config = configMap[name] || {};
    const type = (0, _produceType2.default)(fieldAST.type, types);
    const args = buildFieldConfigArgumentMap(fieldAST, types);
    const description = (0, _buildASTSchema.getDescription)(fieldAST);
    const fieldConfig = { type };
    if (Object.keys(args).length) fieldConfig.args = args;
    if ('resolve' in config) fieldConfig.resolve = config.resolve;
    if ('deprecationReason' in config) fieldConfig.deprecationReason = config.deprecationReason;
    if (description) fieldConfig.description = description;
    return _extends({}, map, { [name]: fieldConfig });
  }, {});
}