'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildTypes = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = build;

var _type = require('graphql/type');

var _language = require('graphql/language');

var _buildSchema = require('./buildSchema');

var _buildSchema2 = _interopRequireDefault(_buildSchema);

var _buildScalar = require('./buildScalar');

var _buildScalar2 = _interopRequireDefault(_buildScalar);

var _buildObject = require('./buildObject');

var _buildObject2 = _interopRequireDefault(_buildObject);

var _buildInterface = require('./buildInterface');

var _buildInterface2 = _interopRequireDefault(_buildInterface);

var _buildUnion = require('./buildUnion');

var _buildUnion2 = _interopRequireDefault(_buildUnion);

var _buildEnum = require('./buildEnum');

var _buildEnum2 = _interopRequireDefault(_buildEnum);

var _buildInputObject = require('./buildInputObject');

var _buildInputObject2 = _interopRequireDefault(_buildInputObject);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const buildMap = {
  [_language.Kind.SCALAR_TYPE_DEFINITION]: _buildScalar2.default,
  [_language.Kind.OBJECT_TYPE_DEFINITION]: _buildObject2.default,
  [_language.Kind.INTERFACE_TYPE_DEFINITION]: _buildInterface2.default,
  [_language.Kind.UNION_TYPE_DEFINITION]: _buildUnion2.default,
  [_language.Kind.ENUM_TYPE_DEFINITION]: _buildEnum2.default,
  [_language.Kind.INPUT_OBJECT_TYPE_DEFINITION]: _buildInputObject2.default
};

const buildTypes = exports.buildTypes = (documentAST, configMap = {}, typeDependencies = []) => {
  const derived = documentAST.definitions.map(definition => {
    const buildDefinition = buildMap[definition.kind];
    if (!buildDefinition) return false;
    return buildDefinition(definition, configMap[definition.name.value], () => [...derived, ...typeDependencies]);
  }).filter(x => x);
  return [...derived, ...typeDependencies];
};

function build(source, config = {}, typeDeps = []) {
  if (typeof source !== 'string') throw new Error(`Expected a string but got ${ source }`);
  if (!(config instanceof Object)) throw new Error(`Expected an object but got ${ config }`);
  if (!Array.isArray(typeDeps)) throw new Error(`Expected an array but got ${ typeDeps }`);

  const documentAST = (0, _language.parse)(source);
  const { definitions } = documentAST;

  // if there is only one type definition and it's not named Query, build the type and return it
  if (definitions.length === 1 && definitions[0].name.value !== 'Query' && Object.keys(buildMap).includes(definitions[0].kind)) {
    const type = buildMap[definitions[0].kind](definitions[0], config, typeDeps);
    if (typeDeps.some(({ name }) => name === type.name)) {
      throw new Error(`Duplicate type "${ type.name }"`);
    }
    return type;
  }

  const types = buildTypes(documentAST, config, typeDeps);
  // check for duplicate types
  types.forEach(typeA => {
    if (types.some(typeB => typeA !== typeB && typeA.name === typeB.name)) {
      throw new Error(`Duplicate type "${ typeA.name }"`);
    }
  });

  // try to build the defined schema
  const schemaASTs = definitions.filter(({ kind }) => kind === _language.Kind.SCHEMA_DEFINITION);
  if (schemaASTs.length > 1) throw new Error('Must provide only one schema definition.');
  if (schemaASTs.length === 1) return (0, _buildSchema2.default)(schemaASTs[0], config, types);

  // try to build an inferred schema
  const QueryType = types.find(({ name }) => name === 'Query');
  if (QueryType) {
    return new _type.GraphQLSchema({
      query: QueryType,
      mutation: types.find(({ name }) => name === 'Mutation'),
      subscription: types.find(({ name }) => name === 'Subscription'),
      types: config.types,
      directives: config.directives
    });
  }

  // return a type map
  return types.reduce((map, type) => _extends({}, map, { [type.name]: type }), {});
}