'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveThunk = exports.buildWrappedType = exports.getNamedTypeAST = undefined;
exports.default = produceType;

var _type = require('graphql/type');

var _language = require('graphql/language');

const { LIST_TYPE, NON_NULL_TYPE } = _language.Kind;

const scalarTypeMap = {
  Int: _type.GraphQLInt,
  Float: _type.GraphQLFloat,
  String: _type.GraphQLString,
  Boolean: _type.GraphQLBoolean,
  ID: _type.GraphQLID
};

const getNamedTypeAST = exports.getNamedTypeAST = typeAST => {
  let namedTypeAST = typeAST;
  while (namedTypeAST.kind === LIST_TYPE || namedTypeAST.kind === NON_NULL_TYPE) {
    namedTypeAST = namedTypeAST.type;
  }
  return namedTypeAST;
};

const buildWrappedType = exports.buildWrappedType = (innerType, typeAST) => {
  if (typeAST.kind === LIST_TYPE) {
    return new _type.GraphQLList(buildWrappedType(innerType, typeAST.type));
  }
  if (typeAST.kind === NON_NULL_TYPE) {
    return new _type.GraphQLNonNull(buildWrappedType(innerType, typeAST.type));
  }
  return innerType;
};

const resolveThunk = exports.resolveThunk = thunk => typeof thunk === 'function' ? thunk() : thunk;

function produceType(typeAST, types) {
  const namedTypeAST = getNamedTypeAST(typeAST);
  const typeName = namedTypeAST.name.value;
  if (scalarTypeMap[typeName]) return buildWrappedType(scalarTypeMap[typeName], typeAST);
  const maybeType = resolveThunk(types).find(({ name }) => name === typeName);
  if (maybeType) return buildWrappedType(maybeType, typeAST);
  throw new Error(`Type "${ typeName }" not found.`);
}